// 004 u_c_widget_17
// 17 oct 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   John COLIBRI  http://www.jcolibri.com

(*$r+*)

unit u_c_widget_17;
  interface
    uses Classes;

    type c_widget= Class; // Forward

         // -- abstract
         c_dialog_director= Class
                              procedure show_dialog; Virtual; Abstract;
                              procedure widget_changed(p_c_widget: c_widget); Virtual; Abstract;
                              procedure _create_widget(p_c_widget: c_widget); Virtual; Abstract;
                            end; // c_dialog_director

         t_mouse_event= (e_click);

         // -- abstract
         c_widget= class
                     _m_c_dialog_director: c_dialog_director;

                     Constructor create_widget(p_c_dialog_director: c_dialog_director);
                     procedure changed; Virtual;
                     procedure handle_mouse_event(p_mouse_event: t_mouse_event); Virtual;
                     // ...
                     Destructor Destroy; Override;
                   end; // c_widget

         c_list_box_widget= class(c_widget)
                              Constructor create_list_box_widget(p_c_dialog_director: c_dialog_director);
                              function f_get_selection: String;
                              procedure set_list(p_c_string_list: tStringList);
                              procedure handle_mouse_event(p_mouse_event: t_mouse_event); Override;
                              // ...
                              Destructor Destroy; Override;
                            end; // c_list_box_widget

         c_edit_widget= class(c_widget)
                          Constructor create_edit_widget(p_c_dialog_director: c_dialog_director);
                          procedure set_text(p_string: String);
                          function f_get_text: String;
                          procedure handle_mouse_event(p_mouse_event: t_mouse_event); Override;
                          // ...
                          Destructor Destroy; Override;
                        end; // c_edit_widget

         c_button_widget= class(c_widget)
                            Constructor create_button_widget(p_c_dialog_director: c_dialog_director);
                            procedure set_text(p_string: String);
                            procedure handle_mouse_event(p_mouse_event: t_mouse_event); Override;
                            // ...
                            Destructor Destroy; Override;
                          end; // c_button_widget

         c_font_dialog_director= Class(c_dialog_director)
                                   _m_c_ok_button: c_button_widget;
                                   _m_c_cancel_button: c_button_widget;
                                   _m_c_font_list_box: c_list_box_widget;
                                   _m_font_name_edit: c_edit_widget;

                                   constructor create_font_dialog_director;
                                   procedure show_dialog; Virtual;
                                   procedure widget_changed(p_c_widget: c_widget); Override;
                                   procedure _create_widget(p_c_widget: c_widget); Override;
                                   Destructor Destroy; Override;
                                 end; // c_font_dialog_director

  implementation

    // -- c_widget

    Constructor c_widget.create_widget(p_c_dialog_director: c_dialog_director);
      begin
      end; // create_widget

    procedure c_widget.changed;
      begin
        _m_c_dialog_director.widget_changed(Self);
      end; // changed

    procedure c_widget.handle_mouse_event(p_mouse_event: t_mouse_event);
      begin
      end; // handle_mouse_event

    Destructor c_widget.Destroy; 
      begin
      end; // Destroy

    // -- c_list_box_widget

    Constructor c_list_box_widget.create_list_box_widget(p_c_dialog_director: c_dialog_director);
      begin
      end; // create_list_box_widget

    function c_list_box_widget.f_get_selection: string;
      begin
      end; // f_get_selection

    procedure c_list_box_widget.set_list(p_c_string_list: tstringlist);
      begin
      end; // set_list

    procedure c_list_box_widget.handle_mouse_event(p_mouse_event: t_mouse_event); 
      begin
      end; // handle_mouse_event

    Destructor c_list_box_widget.Destroy; 
      begin
      end; // Destroy

    // -- c_edit_widget

    Constructor c_edit_widget.create_edit_widget(p_c_dialog_director: c_dialog_director);
      begin
      end; // create_edit_widget

    procedure c_edit_widget.set_text(p_string: string);
      begin
      end; // set_textt

    function c_edit_widget.f_get_text: string;
      begin
      end; // f_get_text

    procedure c_edit_widget.handle_mouse_event(p_mouse_event: t_mouse_event); 
      begin
      end; // handle_mouse_event

    Destructor c_edit_widget.Destroy; 
      begin
      end; // Destroy

    // -- c_button_widget

    Constructor c_button_widget.create_button_widget(p_c_dialog_director: c_dialog_director);
      begin
      end; // create_button_widget

    procedure c_button_widget.set_text(p_string: string);
      begin
      end; // set_text

    procedure c_button_widget.handle_mouse_event(p_mouse_event: t_mouse_event); 
      begin
        changed;
      end; // handle_mouse_event

    Destructor c_button_widget.Destroy;
      begin
      end; // Destroy

    // -- c_font_dialog_director

    constructor c_font_dialog_director.create_font_dialog_director;
      begin
        inherited Create;

        _m_c_ok_button:= c_button_widget.create_button_widget(Self);
        _m_c_cancel_button:= c_button_widget.create_button_widget(Self);;
        _m_c_font_list_box:= c_list_box_widget.create_list_box_widget(Self);;
        _m_font_name_edit:= c_edit_widget.create_edit_widget(Self);;

        // -- fill the widgets (the font names etc)
      end; // create_font_dialog_director

    procedure c_font_dialog_director.show_dialog; 
      begin
      end; // show_dialog

    procedure c_font_dialog_director.widget_changed(p_c_widget: c_widget); 
      begin
        if p_c_widget= _m_c_font_list_box
          then _m_font_name_edit.set_text(_m_c_font_list_box.f_get_selection)
          else
            if p_c_widget= _m_c_ok_button
              then // apply the font changes
              else
                if p_c_widget= _m_c_cancel_button
                  then // ...
      end; // widget_changed

    procedure c_font_dialog_director._create_widget(p_c_widget: c_widget);
      begin
      end; // _create_widget

    Destructor c_font_dialog_director.Destroy;
      begin
      end; // Destroy



    begin // u_c_widget
    end. // u_c_widget
